#ifndef _CACHE_H
#define _CACHE_H

#include <SpatialIndex.h>
#include <list>
#include <Tools.h>

namespace cache
{
	//Forward References
	class ReplacementPolicy;
	class Mapper;
	class CacheSearcher;
	class Brush;
	class ICacheEntry;
	class IIndex;

	class Cache
	{
		//Friends
		friend class Mapper;
		friend class ReplacementPolicy;

	public:

		Cache(long sizeOfCache,
			ICacheEntry* buffer,
			IIndex* index);

		void addCacheEntry(ICacheEntry& cacheEntry);

		/**
		* Empties the cacheEntry, prepares it for containing a new object
		* \param &cacheEntry 
		*/
		void emptyCacheEntry(ICacheEntry &cacheEntry);

		std::list<ICacheEntry*>& searchNode(Brush& brush);	


		/**
		* Deletes the items from the cache
		* \return 
		*/
		virtual ~Cache();	

		/**
		* This method is used by the ReplacementPolicy
		* \param replacmentPolicy 
		*/
		void setReplacementPolicy(ReplacementPolicy* replacementPolicy);

	private:
		//This points to the buffer
		ICacheEntry *buffer;
		//Store the size of the cache
		long sizeOfCache;
		//Stores the pointer to the index
		IIndex *indexPtr;		
		//Stores the pointer to the current Replacement Policy
		ReplacementPolicy* replacementPolicy;
		///Stores the current Brush
		Brush *currentBrush;	
	};
	/************************************************************************/
	/* This defines the interface for all the index structures
	The index structure should implement this interface and the
	This interface will be used directtly the cache.
	/************************************************************************/
	class IIndex
	{
	public:
		/**
		* Inserts the cache entry into the index.
		* Called when a new 
		* \param &cacheEntry 
		* \return 
		*/
		virtual void insertData(ICacheEntry& cacheEntry) = 0;

		/**
		* Deletes the dataitem from the index
		* \param iShape 
		* \param id 
		* \return 
		*/
		virtual void deleteData(Tools::Geometry::IShape* iShape,unsigned long id)=0;

		/**
		* Deletes the data item from the br
		* \param brush 
		* \param resultList 
		* \return 
		*/
		virtual std::list<ICacheEntry*>& search(Brush& brush)=0;

		/**
		* Deletes the index structure
		* \return 
		*/
		virtual ~IIndex(){};

	};


	class RTreeIndex : public IIndex,public SpatialIndex::IQueryStrategy
	{
	public:
		RTreeIndex(double fillFactor, 
			unsigned long indexCapacity,			  
			unsigned long leafCapacity,
			unsigned numberOfDimensions);
		/*
		 *\param currentBrush	
		 */
		ICacheEntry* findFarthestNode(Brush *currentBrush);

		void getNextEntry(const SpatialIndex::IEntry& previouslyFetched, 
						  long& nextEntryToFetch, 
						  bool& bFetchNextEntry);

		RTreeIndex();
		/**
		* Copy constructor undefined
		* \param &rTreeIndex 
		* \return 
		*/
		RTreeIndex(const RTreeIndex &rTreeIndex);

		/**
		* = operator undefined
		* \param rTreeIndex 
		* \return 
		*/
		RTreeIndex& operator =(const RTreeIndex& rTreeIndex);

		/**
		* Destroys the R Tree index structure
		* \return 
		*/
		~RTreeIndex();

		std::list<ICacheEntry*>& search(Brush& brush);

		void deleteData(Tools::Geometry::IShape* iShape,unsigned long id);

		void RTreeIndex::insertData(ICacheEntry& cacheEntry);

	private:
		//Stores the result List
		std::list<ICacheEntry*> resultList;
		//Stores the cache searacher
		CacheSearcher* cacheSearcher;
		//Stores the index identifier
		long indexIdentifier;	
		//Has to maintain the R Tree
		SpatialIndex::ISpatialIndex *rTree;
		//Stores the reference to the storage manager
		SpatialIndex::IStorageManager* memoryStorageManager;			
	};
	/*
	*	The implementation of the index might not be very efficient
	*  But we are using this mainly to do experiemts.
	*/
	class SequentialIndex : public IIndex
	{
	public:
		/**
		* Creates the sequential index structure
		* \param sizeOfCache 
		* \return 
		*/
		SequentialIndex(unsigned long sizeOfCache);

		/**
		* Copy constructor undefined
		* \param sequentialIndex 
		* \return 
		*/
		SequentialIndex(const SequentialIndex& sequentialIndex);

		/**
		* Operator = undefined
		* \param squentialIndex 
		* \return 
		*/
		SequentialIndex& operator =(SequentialIndex& squentialIndex);

		/**
		* Deletes the data items
		* \param *iShape 
		* \param id 
		*/
		void deleteData(Tools::Geometry::IShape *iShape,unsigned long id);

		/**
		* Inserts the cache entry into the index.
		* Called when a new 
		* \param &cacheEntry 
		* \return 
		*/
		void SequentialIndex::insertData(ICacheEntry& cacheEntry);

		/*
		*	Destroys the sequential index
		*/
		SequentialIndex::~SequentialIndex();

		/**
		* Does a sequential scan over the complete data set in the main memory
		* \param &brush 
		* \return 
		*/
		std::list<ICacheEntry*>& SequentialIndex ::search(Brush &brush);

	private:
		//Stores the pointers to cacheentries inside the cache.
		std::vector<ICacheEntry*> seqList;
		//Stores the resultList
		std::list<ICacheEntry*> resultList;
		//Stores the size of the cache
		unsigned long sizeOfCache;
	};

}
#endif //Remove the cache
