////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  XmdvTool - A program for Multivariate Data Display
//
//  Copyright 1994-2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
////////////////////////////////////////////////////////////// COPYRIGHT //

///////////////////////////////////////////////////////////////////////////
/// \file ColorMapEditor.h
/// \brief 
/// \date 7 Feb 2004
/// \author Anilkumar Patro
///////////////////////////////////////////////////////////////////////////

#ifndef __COLORMAPEDITOR_H__
#define __COLORMAPEDITOR_H__

// SYSTEM INCLUDES
//
#include <cassert>
#include <iosfwd>
#include <map>
#include <string>

// PROJECT INCLUDES
//
#include <xmdv/CDisplay.h>
#include <xmdv/colorramp.h>

// FORWARD REFERENCES
//
class colormap;

///
/// Color map editor GUI
///
/// \date 02-07-2004
/// \author anilkpatro
/// \version 1.0
/// \see colormap, CDisplay
///
class CColorMapEditor : public CDisplay
{
public:		// types

	/// Modes of the colormap
	enum CMAPMode {
		MODE_RGBA,	///< Use the RGBA color model
		MODE_HSVA,	///< Use the HSVA color model
		MODE_GREY,	///< Use the RGBA color model, but RGB components are locked
	};

public:		// methods

// LIFECYCLE
//
	/// 
	/// Constructor initialized with an optional colormap
	/// 
	/// \param *map the color map
	///
	CColorMapEditor(colormap *map = NULL);

	/// 
	/// Destructor
	///
	~CColorMapEditor(void);

// OPERATIONS
//
	// -- CDisplay implementation
	
	/// 
	/// Set parameters used for drawing, draw the display showing the
	/// color map, and the ramp points.
	/// 
	void		Draw();

	//event handlers

	/// 
	/// Update the status. And move points/tangents on the ramp if
	/// selected.
	/// 
	/// \param x Current mouse x position
	/// \param y Current mouse y position
	///
	void		InputMouseMotion(int x, int y);

	/// 
	/// See if the curve is under the clicked mouse location. If yes,
	/// then determine, if it's a point, or tangent point or a new point
	/// on the curve.
	/// 
	/// \param button Which button?
	/// \param x Current mouse x position
	/// \param y Current mouse y position
	///
	void		InputButtonPress(int button, int x, int y);

	/// 
	/// If mouse was pressed earlier, then move out of pressed state.
	/// 
	/// \param button Which button?
	/// \param x Current mouse x position
	/// \param y Current mouse y position
	///
	void		InputButtonRelease(int button, int x, int y);

	/// 
	/// Remove the currently selected point on the graph
	/// 
	void		RemoveCurrPt();

	/// 
	/// Load the color map values from a stream
	/// 
	/// \param inputstream load color map from this stream
	/// \return status (true = loaded correctly)
	///
	bool		Load(std::istream &inputstream);

	/// 
	/// Save the current color map to the specified stream
	/// 
	/// \param outputstream stream to serialize the color map to
	/// \return status (true = saved successfully)
	///
	bool		Save(std::ostream &outputstream);

	void		EditColorMap(const std::string& whichmap);

// ACCESS
//
	/// 
	/// Access to the current component of the color
	/// 
	/// \return current component
	///
	int			GetCurrCmpnt() const;
	/// 
	/// Set the current component of color to the specified component
	/// 
	/// \param i component of color (1 <= i <= 4)
	///
	void		SetCurrCmpnt(int i);

	/// 
	/// Access to the RampFunction being used on the currently selected 
	/// point.
	/// 
	/// \return Function being used
	/// \see ColorRamp::RampFunction
	///
	ColorRamp::RampFunction	GetCurrPtMode() const;
	/// 
	/// Set the RampFunction being used on the currently selected point.
	/// Cannot be used on the last point.
	/// 
	/// \param fn Function to be used
	/// \see ColorRamp::RampFunction
	///
	void		SetCurrPtMode(ColorRamp::RampFunction fn);

	/// 
	/// Get the current mode of the colormap
	/// 
	/// \return mode
	///
	CMAPMode	GetCMAPMode() const;
	/// 
	/// Set the current mode of the colormap
	/// 
	/// \param mode mode of the colormap
	///
	void		SetCMAPMode(CMAPMode mode);

private:	// members

	colormap	*m_cmap;			///< color map being edited
	colormap	*m_currcmap;		///< color vales for the current cmpnt
	ColorRamp::PointListIter	m_currpoint;	///< current point
	int			m_currcmpnt;		///< current component of the color
	bool		m_leftdown;			///< maintains state if left button was clicked
	bool		m_requireupdate;	///< does ramp require update on next draw?
	CMAPMode	m_mode;				///< current mode of editor
	bool		m_modified;			///< has the color map been modified?

	/// Enumerated type to determine type of point on the curve under the
	/// mouse
	enum Type { 
		UNKNOWN	= -1,				///< point not on curve
		POINT = 1,					///< existing point on the curve
		CURVE = 2,					///< point on curve, but new
		LEFT_TAN = 3,				///< point of the left tangent
		RIGHT_TAN = 4,				///< point of the right tangent
	};
	Type		m_currType;			///< current type of point selected

	// Forward reference
	// Type which defines the picked object
	struct		PickObject;

private:	// methods

	/// 
	/// Determine the object under the mouse on mouse button click.
	/// 
	/// \param x Current mouse x position
	/// \param y Current mouse y position
	/// \param *obj Picked object
	///
	void		Pick(float x, float y, PickObject *obj);

};

//-------------------------------------------------------------------------
inline int CColorMapEditor::GetCurrCmpnt() const
{
	return m_currcmpnt;
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
inline ColorRamp::RampFunction CColorMapEditor::GetCurrPtMode() const
{
	return (*m_currpoint)->mode;
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
inline CColorMapEditor::CMAPMode CColorMapEditor::GetCMAPMode() const
{
	return m_mode;
}
//-------------------------------------------------------------------------

#endif //__COLORMAPEDITOR_H__
