
#include <xmdv/CGlyphTool.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/CFlatGlyph.h>
#include <xmdv/brush.h>
#include <xmdv/ogl.h>
#include <xmdv/okcdata.h>

static int axis = -1;
static double mouse_angle;
static int resize_which = -1;

const double R = 0.25; // Center width

CGlyphTool::CGlyphTool(CFlatDisplayInformation* flat_info) : CFlatGlyph(flat_info)
{
}


void CGlyphTool:: Draw()
{
    if (!g_globals.dataset) return;
    
    canvas->MakeCurrent();

    // Setup window parameters.
    //
    int W, H;
    canvas->GetSize(&W,&H);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    const double BORDER = 0.25;
    const double L = 1 + R + BORDER;
    glViewport( 0, 0, W, H);
    glOrtho(-L,L,-L,L,-L,L);

    canvas->SetBackground(interface_color[BG]);
    canvas->Clear();

    // Precompute table of values
    //
    int N = g_globals.dataset->dims;
    int i;
    double angle = 0;

    ARR<double> cosa(N), sina(N), lo(N), hi(N);
    
    for (i=0;i<N;i++) {
        cosa[i] = cos(angle);
        sina[i] = sin(angle);
        angle += 2.*M_PI*g_globals.dataset->one_over_dims;
        lo[i] = (g_globals.curr_brush->Min(i)-g_globals.dataset->org_dim_min[i])/
                (g_globals.dataset->org_dim_max[i] - g_globals.dataset->org_dim_min[i]) + R;
        hi[i] = (g_globals.curr_brush->Max(i)-g_globals.dataset->org_dim_min[i])/
                (g_globals.dataset->org_dim_max[i] - g_globals.dataset->org_dim_min[i]) + R;
    }

    // Paint the current brush
    CFlatGlyph::DrawSingleBrush(g_globals.curr_brush);

    // Draw the brush
    //
    canvas->SetForeground(interface_color[BRUSH1+g_globals.curr_brush->num]);

    ARR<Vec2> pts(4);
    pts[0].X = cosa[N-1] * hi[N-1];
    pts[1].X = cosa[N-1] * lo[N-1];

    pts[0].Y = sina[N-1] * hi[N-1];
    pts[1].Y = sina[N-1] * lo[N-1];

    for (i=0; i<N; i++ )
    {
        pts[3].X = cosa[i] * hi[i];
        pts[3].Y = sina[i] * hi[i];

        pts[2].X = cosa[i] * lo[i];
        pts[2].Y = sina[i] * lo[i];

        canvas->FillPolygon(pts);
        
        pts[0] = pts[3];
        pts[1] = pts[2];
    }

    // Draw the axes, labels
    //
    Vec2 size = canvas->GetDataFontSize("X");

    for (i=0; i<N; i++) 
    {
        canvas->SetForeground(interface_color[GRID1]);
        canvas->DrawLine((float)(R*cosa[i]),
						(float)(R*sina[i]),
						(float)((1+R)*cosa[i]),
						(float)((1+R)*sina[i]));

        double angle = i*2.*M_PI*g_globals.dataset->one_over_dims;

        double dx;
        if (angle<M_PI) dx = -angle/M_PI;
        else dx = -(2*M_PI - angle)/M_PI;
        
        Vec2 pos;
        pos.X = (1.01+R)*cosa[i] + dx*BORDER*.8;
        pos.Y = (1.01+R)*sina[i] - (size.Y/2.) - (size.Y/2.) * sin(angle+M_PI);

        canvas->SetForeground(interface_color[TEXT]);
        canvas->DrawConfinedString(pos.X,pos.Y,g_globals.dataset->names[i],BORDER);
    }

    // Draw the center loop
    //
    canvas->SetForeground(interface_color[GRID1]);
    for (i=1; i<N; i++) {
        canvas->DrawLine((float)(R*cosa[i]),
						(float)(R*sina[i]),
						(float)(R*cosa[i-1]),
						(float)(R*sina[i-1]) );
    }
    canvas->DrawLine((float)(R*cosa[N-1]),
					(float)(R*sina[N-1]),
					(float)(R*cosa[0]),
					(float)(R*sina[0]) );
}

static void glyphtoolResize(grCanvas *canvas, int x, int y)
{
    if (axis<0) return;

    // Map mouse location to data coordinates 
    Vec2 p = canvas->MapScreenToData(Vec2(x,y));

    // Find angle of axis 
    double angle = axis * (2*M_PI*g_globals.dataset->one_over_dims);

    // Calculate distance from center 
    double dot = cos(angle)*p.X + sin(angle)*p.Y - R;
    if (dot>1) dot = 1;
    else if (dot<0) dot = 0;

    // Calculate new position and bound it 
    double new_pos = g_globals.dataset->org_dim_min[axis]+dot*(g_globals.dataset->org_dim_max[axis]-g_globals.dataset->org_dim_min[axis]);
    new_pos = MIN2(new_pos, g_globals.dataset->org_dim_max[axis]);
    new_pos = MAX2(new_pos, g_globals.dataset->org_dim_min[axis]);

    double lo = g_globals.curr_brush->Min(axis);
    double hi = g_globals.curr_brush->Max(axis);

    if (resize_which<0) {
        if (fabs(new_pos-lo) < fabs(new_pos-hi)) {
            resize_which = 0;
        } else {
            resize_which = 1;
        }
    }

    if (resize_which==0) lo = MIN2(new_pos,hi);
    else hi = MAX2(new_pos, lo);

    g_globals.curr_brush->SetMinMax(axis,lo,hi);

    appBrushBoundaryCaption(canvas, axis);
}

static void glyphtoolMove(grCanvas *canvas, int x, int y)
{
    if (axis<0) return;

    // Map mouse location to data coordinates 
    Vec2 p = canvas->MapScreenToData(Vec2(x,y));

    // Find angle of axis 
    double angle = axis * (2*M_PI*g_globals.dataset->one_over_dims);

    // Calculate distance from center 
    double dot = cos(angle)*p.X + sin(angle)*p.Y - R;
    if (dot>1) dot = 1;
    else if (dot<0) dot = 0;

    double new_pos = g_globals.dataset->org_dim_min[axis]+dot*(g_globals.dataset->org_dim_max[axis]-g_globals.dataset->org_dim_min[axis]); 
    new_pos = MIN2(new_pos, g_globals.dataset->org_dim_max[axis]-g_globals.curr_brush->Size(axis)/2.);
    new_pos = MAX2(new_pos, g_globals.dataset->org_dim_min[axis]+g_globals.curr_brush->Size(axis)/2.);

    g_globals.curr_brush->SetPos(axis, new_pos);

    appBrushBoundaryCaption(canvas, axis);
}

void glyphtoolMotionNotify(grCanvas *canvas, int x, int y)
{
    if (canvas->button[0]==true) {
        glyphtoolResize(canvas,x,y);
        canvas->Refresh();
    } else
    if (canvas->button[1]==true) {
        glyphtoolMove(canvas,x,y);
        canvas->Refresh();
    }
    else
    {
        char caption[1000]="\0";
        sprintf(caption,"Use left mouse button to resize brush; Use middle mouse button to move brush");
        appShowCanvasMessage(Togl_Interp(canvas->GetWidget()),caption, canvas->name);
    }
}

void glyphtoolButtonPress(grCanvas *canvas, int button, int x, int y)
{
    if (!g_globals.dataset) return;
    
    Vec2 pick = canvas->MapScreenToData(Vec2(x,y));

    // Find the angle of the line from the center to the mouse 
    // position 
    mouse_angle = atan2(pick.Y, pick.X);

    // Add a half wedge to the angle 
    double angle = mouse_angle + ((2.*M_PI)/(double)g_globals.dataset->dims)/2.;

    // Wrap angle around if it is negative 
    if (angle < 0.) angle += 2.*M_PI;

    // Map angle to an axis 
    axis = (int)(g_globals.dataset->dims*(angle/(2.*M_PI)));

    // Sanity check 
    if (axis < 0 || axis >= g_globals.dataset->dims) {
        axis = -1;
        return;
    }

    glyphtoolMotionNotify(canvas,x,y);
}

void glyphtoolButtonRelease(grCanvas *canvas, int button, int x, int y)
{
    if (!g_globals.dataset || axis<0) return;

    appCanvasBrushHasChanged(Togl_Interp(canvas->GetWidget()));
    appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
    axis = -1;
    resize_which = -1;
}


void CGlyphTool::InputMouseMotion(int x, int y)
{
	glyphtoolMotionNotify(canvas, x, y);
}

void CGlyphTool::InputButtonPress(int button, int x, int y)
{
	glyphtoolButtonPress(canvas, button, x, y);
}

void CGlyphTool::InputButtonRelease(int button, int x, int y)
{
	glyphtoolButtonRelease(canvas, button, x, y);
}

