///////////////////////////////////////////////////////////////////////////////
/// \file ColorMapEditor.cpp
/// \brief 
/// \date 4 Feb 2004
/// \author Anilkumar Patro
///////////////////////////////////////////////////////////////////////////////

// SYSTEM INCLUDES
//
#include <sstream>
#include <iomanip>

// PROJECT INCLUDES
//
#include <xmdv/ColorMapEditor.h>
#include <xmdv/ogl.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/colorramp.h>

// DEFINES
//
#define	LEFT		(-15.f)
#define	RIGHT		(MAXNDX+20.f)
#define	BOTTOM		(-25.f)
#define	TOP			(MAXNDX+60.f)

#define	SUP(a,b)	( ((a)>(b)) ? (a) : (b) )
#define	INF(a,b)	( ((a)<(b)) ? (a) : (b) )

// TYPES
//
struct CColorMapEditor::PickObject
{
	Type						type;
	int							cmpnt, 
								ndx;
	ColorRamp::PointListIter	upoint;
};

// GLOBALS
//
float s_RGBColorRamps[][2][5]	= {
	{{ 0, 0, 0, 0, 255 }, { 255, 255, 0, 0, 255 }},
	{{ 0, 0, 0, 0, 255 }, { 255, 0, 255, 0, 255 }},
	{{ 0, 0, 0, 0, 255 }, { 255, 0, 0, 255, 255 }},
	{{ 0, 0, 0, 0, 255 }, { 255, 255, 255, 255, 255 }}, 
};

float s_HSVColorRamps[][2][5]	= {
	{{ 0, 0, 255, 255, 255 }, { 255, 255, 255, 255, 255 }},
	{{ 0, 0, 0, 255, 255 }, { 255, 0, 255, 255, 255 }},
	{{ 0, 0, 255, 0, 255 }, { 255, 0, 255, 255, 255 }},
	{{ 0, 0, 0, 0, 255 }, { 255, 0, 0, 255, 255 }}, 
};

float s_GREYColorRamps[][2][5]	= {
	{{ 0, 0, 0, 0, 255 }, { 255, 255, 255, 255, 255 }}, 
};

// FORWARD REFERENCES
//
void appSetCMAPCurrCmpnt(Tcl_Interp *interp, int cmpnt);
void appSetCMAPCurrPtMode(Tcl_Interp *interp, ColorRamp::RampFunction fn);
void appDisableFunctionButtons(Tcl_Interp *interp);
void appDisableDeleteButton(Tcl_Interp *interp);
void appEnableDeleteButton(Tcl_Interp *interp);

//-----------------------------------------------------------------------------
CColorMapEditor::CColorMapEditor(colormap *map /*= NULL*/)
{
	if (!map)
		m_cmap = new colormap("", new ColorRamp(CMAPSIZE));
	else
		m_cmap = map;
	m_cmap->InitTex();
	m_currcmpnt = 0;
	m_currpoint = m_cmap->GetColorRamp()->Begin(m_currcmpnt);
	m_currcmap = new colormap("Side", new ColorRamp(CMAPSIZE, 2,
								s_RGBColorRamps[m_currcmpnt]));
	m_leftdown = false;
	m_requireupdate = false;
	m_currType = UNKNOWN;
	m_mode = MODE_RGBA;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
CColorMapEditor::~CColorMapEditor(void)
{
//	delete m_cmap;
	delete m_currcmap;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::Draw()
{
	const int MAXNDX				= m_cmap->Len() - 1;

	ColorRamp::RampPoint *curPoint	= *m_currpoint;
	float ndx						= curPoint->point.x;
	float val						= curPoint->point.y;
	int numCmpnts					= 4;

	ColorRamp *ramp	= m_cmap->GetColorRamp();
	// if curves require update, then update them
	if (m_requireupdate) {
		if (m_currpoint == ramp->Begin(m_currcmpnt))
			ramp->Update(m_currcmpnt, m_currpoint+1);
		else
			ramp->Update(m_currcmpnt, m_currpoint);
		m_requireupdate = false;
	}

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(LEFT, RIGHT, BOTTOM, TOP, -1.0f, 1.0f);

	glClearColor(0.8f, 0.8f, 0.8f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	//	glShadeModel( GL_FLAT );

	glMatrixMode(GL_MODELVIEW);

	if (m_mode == MODE_RGBA) {
		m_cmap->SetColorModel(colormap::MODEL_RGB);
		m_currcmap->SetColorModel(colormap::MODEL_RGB);
		m_currcmap->SetColorRamp(new ColorRamp(CMAPSIZE, 2, 
											s_RGBColorRamps[m_currcmpnt]));
	} else if (m_mode == MODE_HSVA) {
		m_cmap->SetColorModel(colormap::MODEL_HSV);
		m_currcmap->SetColorModel(colormap::MODEL_HSV);
		m_currcmap->SetColorRamp(new ColorRamp(CMAPSIZE, 2, 
											s_HSVColorRamps[m_currcmpnt]));
	} else /*if (m_mode == MODE_GREY)*/ {
		m_cmap->SetColorModel(colormap::MODEL_GREY);
		m_currcmap->SetColorModel(colormap::MODEL_GREY);
		m_currcmap->SetColorRamp(new ColorRamp(CMAPSIZE, 2, 
											s_GREYColorRamps[0]));
	}

	// Color Map
	m_cmap->DrawColorBar(-1.0f, (float)MAXNDX+40, 
		(float)MAXNDX-1, (float)MAXNDX+40, 40.0f);
	m_currcmap->DrawColorBar((float)MAXNDX+10, 0.0f, 
							 (float)MAXNDX+10, (float)MAXNDX, 15.0f);

	// Graph
	// - background
	glColor3f(0., 0., 0.);
	glBegin(GL_TRIANGLE_STRIP); {
		glVertex2f(0.0f, 0.0f);
		glVertex2f((float)MAXNDX, 0.0f);
		glVertex2f(0.0f, (float)MAXNDX);
		glVertex2f((float)MAXNDX, (float)MAXNDX);
	}
	glEnd();

	// - ticks
	canvas->SetForeground(interface_color[TEXT]);
	std::ostringstream oss;
	float t = 0.f;
	glLineWidth(2.0f);
	glBegin(GL_LINES); {
		// y
		for (t = 1.0f; t <= (float)MAXNDX; t += 64.0f) {
			glVertex2f(-5.0f, t);
			glVertex2f(0.0f, t);
			oss << (int)t;
//			canvas->DrawConfinedString(-10.0f, t, 
//					oss.str().c_str(), oss.str().size());
		} 
		glVertex2f(-5.0f, (float)MAXNDX);
		glVertex2f(0.0f, (float)MAXNDX);
		glRasterPos2f(-10.0f, (float)MAXNDX);
		canvas->DrawString("255", 3);

		// x
		for (t = 0.0f; t <= (float)MAXNDX; t += 64.0f) {
			glVertex2f(t, -5.0f);
			glVertex2f(t, 1.0f);
		} 
		glVertex2f((float)MAXNDX, -5.0f);
		glVertex2f((float)MAXNDX, 1.0f);
	}
	glEnd();
	glLineWidth(1.0f);


	// - curves
	int priority[4], i;
	for (i = 0; i < m_currcmpnt; i++)
		priority[i] = i;
	for (i = m_currcmpnt + 1; i < 4; i++)
		priority[i - 1] = i;
	priority[3] = m_currcmpnt;

	int cmpnt	= priority[0];
	for (i = 0; i < 4; ++i)
	{
		cmpnt = priority[i];
		if ((m_mode == MODE_GREY) && ((cmpnt == 1) || (cmpnt == 2)))
			continue;

		switch (cmpnt)
		{
		case 0: glColor3f(1.0f, 0.0f, 0.0f); break;
		case 1: glColor3f(0.0f, 1.0f, 0.0f); break;
		case 2: glColor3f(0.0f, 0.0f, 1.0f); break;
		case 3: glColor3f(1.0f, 1.0f, 1.0f); break;
		}

		for (int j = 0; j < MAXNDX; j++)
		{
			double cj, cjp1;
			cj = MAXNDX * ramp->Evaluate(j, cmpnt);
			cjp1 = MAXNDX * ramp->Evaluate(j + 1, cmpnt);

			glBegin(GL_LINES);
			glVertex2f((float) j, (float) cj);
			glVertex2f((float) (j + 1), (float) cjp1);
			glEnd();
		}
	}

	// points
	const int DIMPOINT				= 2;
	ColorRamp::PointListIter iter	= ramp->Begin(m_currcmpnt);

	do
	{
		ColorRamp::RampPoint *upoint= *iter;
		glBegin(GL_TRIANGLE_STRIP);
		glVertex2f(upoint->point.x - DIMPOINT, upoint->point.y - DIMPOINT);
		glVertex2f(upoint->point.x - DIMPOINT, upoint->point.y + DIMPOINT);
		glVertex2f(upoint->point.x + DIMPOINT, upoint->point.y - DIMPOINT);
		glVertex2f(upoint->point.x + DIMPOINT, upoint->point.y + DIMPOINT);
		glEnd();
		// Tangent
		ColorRamp::RampPoint::Tangent *tg	= &upoint->tangent;
		if ( (iter !=ramp->Begin(m_currcmpnt)) ?
			 ((*(iter - 1))->mode == ColorRamp::POLYNOMIAL) : 0)
		{
			glBegin(GL_LINES);
			glVertex2f(tg->x0, tg->y0);
			glVertex2f(upoint->point.x, upoint->point.y);
			glEnd();
			glBegin(GL_LINE_LOOP);
			glVertex2f(tg->x0 - DIMPOINT, tg->y0 - DIMPOINT);
			glVertex2f(tg->x0 - DIMPOINT, tg->y0 + DIMPOINT);
			glVertex2f(tg->x0 + DIMPOINT, tg->y0 + DIMPOINT);
			glVertex2f(tg->x0 + DIMPOINT, tg->y0 - DIMPOINT);
			glEnd();
		}
		if (upoint->mode == ColorRamp::POLYNOMIAL)
		{
			glBegin(GL_LINES);
			glVertex2f(upoint->point.x, upoint->point.y);
			glVertex2f(tg->x1, tg->y1);
			glEnd();
			glBegin(GL_LINE_LOOP);
			glVertex2f(tg->x1 - DIMPOINT, tg->y1 - DIMPOINT);
			glVertex2f(tg->x1 - DIMPOINT, tg->y1 + DIMPOINT);
			glVertex2f(tg->x1 + DIMPOINT, tg->y1 + DIMPOINT);
			glVertex2f(tg->x1 + DIMPOINT, tg->y1 - DIMPOINT);
			glEnd();
		}
	} while (++iter != ramp->End(m_currcmpnt));

	// Highlight current point
	switch (m_currcmpnt)
	{
	case 3:	glColor3f(1., 1., 1.); break;
	case 2: glColor3f(0., 0., 1.); break;
	case 1: glColor3f(0., 1., 0.); break;
	case 0: glColor3f(1., 0., 0.); break;
	}
	glBegin(GL_LINE_LOOP);
	glVertex2f((float) ndx - 2.f * DIMPOINT, (float) val - 2.f * DIMPOINT);
	glVertex2f((float) ndx - 2.f * DIMPOINT, (float) val + 2.f * DIMPOINT);
	glVertex2f((float) ndx + 2.f * DIMPOINT, (float) val + 2.f * DIMPOINT);
	glVertex2f((float) ndx + 2.f * DIMPOINT, (float) val - 2.f * DIMPOINT);
	glEnd();
}
//-----------------------------------------------------------------------------

//event handlers
//-----------------------------------------------------------------------------
void CColorMapEditor::InputMouseMotion(int x, int y)
{
	Vec2 dc			= canvas->MapScreenToData(Vec2(x, y));
	const int MAXNDX= m_cmap->Len() - 1;
	ColorRamp *ramp	= m_cmap->GetColorRamp();

	int ndx			= (int) SUP(INF(dc.X, (float) (m_cmap->Len() - 1)), 0);
	int currndx		= (int) SUP(INF((*m_currpoint)->point.x, 
									(float) (m_cmap->Len() - 1)), 0);
	int cmpnt[4], i=0;

	if (!m_leftdown) {
		for (i=0; i<4; ++i) {
			cmpnt[i] = (int) (MAXNDX * ramp->Evaluate(ndx, i));
		}
	} else {
		for (i=0; i<m_currcmpnt; ++i) {
			cmpnt[i] = (int) (MAXNDX * ramp->Evaluate(currndx, i));
		}
		cmpnt[m_currcmpnt] = (int) (MAXNDX * ramp->Evaluate(ndx, i));
		for (i=m_currcmpnt+1; i<4; ++i) {
			cmpnt[i] = (int) (MAXNDX * ramp->Evaluate(currndx, i));
		}
	}

	std::ostringstream oss;
	oss.width(3); 
	if (m_mode == MODE_RGBA || m_mode == MODE_GREY) {
		oss << ndx << ": R: " << cmpnt[0] << ", G: " << cmpnt[1] 
				   << ", B: " << cmpnt[2] << ", A: " << cmpnt[3];
	} else {
		oss << ndx << ": H: " << cmpnt[0] << ", S: " << cmpnt[1] 
				   << ", V: " << cmpnt[2] << ", A: " << cmpnt[3];
	}
	appShowCanvasMessage(Togl_Interp(canvas->GetWidget()),
						 oss.str().c_str(), canvas->name);

	if (m_leftdown)
	{
		int update;
		switch (m_currType)
		{
		case CURVE:
		case POINT:
			update = ramp->MovePoint(m_currpoint, m_currcmpnt, 
									 (float)dc.X, (float)dc.Y);
			break;

		case LEFT_TAN:
			update = ramp->MoveTangent(m_currpoint, m_currcmpnt, -1, 
									   (float)dc.X, (float)dc.Y);
			break;
		case RIGHT_TAN:
			update = ramp->MoveTangent(m_currpoint, m_currcmpnt, 1, 
									   (float)dc.X, (float)dc.Y);
			break;
		}
		m_requireupdate = (update == 1);
		canvas->Refresh();
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::InputButtonPress(int button, int x, int y)
{
	Vec2 dc			= canvas->MapScreenToData(Vec2(x, y));
	ColorRamp *ramp	= m_cmap->GetColorRamp();

	int oldcmpnt = m_currcmpnt;

	PickObject object;
	if (canvas->button[0] == true && 
		canvas->button[2] == false)
	{
		Pick((float)dc.X, (float)dc.Y, &object);
		m_currType = object.type;
		switch (object.type)
		{
		case CURVE:
			if (ramp->InsertPoint((float)dc.X, (float)dc.Y, object.ndx, 
								   object.cmpnt, object.upoint) != 0)
			{
				m_currpoint = object.upoint;
				m_currcmpnt = object.cmpnt;
			}
			m_leftdown = true;
			canvas->Refresh(true);
			break;

		case POINT:
		case LEFT_TAN:
		case RIGHT_TAN:
			m_currpoint = object.upoint;
			m_currcmpnt = object.cmpnt;
			m_leftdown = true;
			canvas->Refresh(true);
			break;
		}
	}

	if (canvas->button[2] == true &&
		canvas->button[0] == false) 
	{
		Pick((float)dc.X, (float)dc.Y, &object);
		m_currType = object.type;
		switch (object.type)
		{
		case POINT:
			if (object.cmpnt == m_currcmpnt) {
				if (object.upoint == m_currpoint) {
					ramp->RemovePoint(m_currpoint, m_currcmpnt);
					canvas->Refresh(true);
				}
			}
			break;
		}
	}

	if (m_currcmpnt != oldcmpnt)
		appSetCMAPCurrCmpnt(Togl_Interp(canvas->GetWidget()), m_currcmpnt+1);
	appSetCMAPCurrPtMode(Togl_Interp(canvas->GetWidget()), (*m_currpoint)->mode);
	if (m_currpoint == ramp->End(m_currcmpnt)-1)
		appDisableFunctionButtons(Togl_Interp(canvas->GetWidget()));
	if (m_currpoint == ramp->Begin(m_currcmpnt) ||
		m_currpoint == ramp->End(m_currcmpnt)-1)
		appDisableDeleteButton(Togl_Interp(canvas->GetWidget()));
	else 
		appEnableDeleteButton(Togl_Interp(canvas->GetWidget()));
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::InputButtonRelease(int button, int x, int y)
{
	if (m_leftdown)
		m_leftdown = false;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::RemoveCurrPt()
{
	ColorRamp *ramp = m_cmap->GetColorRamp();
	ramp->RemovePoint(m_currpoint, m_currcmpnt);
	appSetCMAPCurrPtMode(Togl_Interp(canvas->GetWidget()), (*m_currpoint)->mode);
	if (m_currpoint == ramp->End(m_currcmpnt)-1)
		appDisableFunctionButtons(Togl_Interp(canvas->GetWidget()));
	if (m_currpoint == ramp->Begin(m_currcmpnt) ||
		m_currpoint == ramp->End(m_currcmpnt)-1)
		appDisableDeleteButton(Togl_Interp(canvas->GetWidget()));
	else 
		appEnableDeleteButton(Togl_Interp(canvas->GetWidget()));
	canvas->Refresh(true);
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
bool CColorMapEditor::Load(std::istream &inputstream)
{
	std::string cmaptoken, cmapname; char bracetoken;
	colormap *cmap;

	inputstream >> cmaptoken >> cmapname >> bracetoken;
	if (cmaptoken == "colormap" && bracetoken == '{')
		cmap = colormap::Load(inputstream, cmapname);
	if (cmap) {
		inputstream >> bracetoken;
		if (bracetoken != '}') {
			return false;
		}
		delete m_cmap;
		m_cmap = cmap;
		return true;
	}

	return false;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
bool CColorMapEditor::Save(std::ostream &outputstream)
{
	outputstream << "colormap" << m_cmap->GetName() << "{";
	bool retValue = colormap::Save(m_cmap, outputstream);
	if (retValue) {
		outputstream << "}";
		return true;
	}

	return false;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::EditColorMap(const std::string& whichmap)
{
	CGlobals::CMapRegistry::iterator iter = g_globals.m_cmaps.find(whichmap);
	if (iter != g_globals.m_cmaps.end()) {
		m_cmap = iter->second;
		SetCurrCmpnt(1);
		m_cmap->InitTex();
		std::string cmapmode = "";
		switch(m_cmap->GetColorModel()) {
		case colormap::MODEL_HSV: cmapmode = "hsva"; break;
		case colormap::MODEL_RGB: cmapmode = "rgba"; break;
		case colormap::MODEL_GREY: cmapmode = "grey"; break;
		default: assert(false);
		}
		Tcl_VarEval(Togl_Interp(canvas->GetWidget()), "SelectCMAPMode ", cmapmode.c_str(), 0);
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::SetCurrCmpnt(int i)
{
	assert(i >= 1);
	assert(i <= 4);

	if (m_currcmpnt == i-1)
		return;

	if (m_mode == MODE_GREY) {
		if (i >= 1 && i <= 3) {
			m_currcmpnt = 0;
		} else {
			m_currcmpnt = i-1;
		}
	} else {
		m_currcmpnt = i-1;
	}
	m_currpoint = m_cmap->GetColorRamp()->Begin(m_currcmpnt);
	appSetCMAPCurrPtMode(Togl_Interp(canvas->GetWidget()), (*m_currpoint)->mode);
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::SetCurrPtMode(ColorRamp::RampFunction fn)
{
	if (fn != (*m_currpoint)->mode) {
		ColorRamp *ramp = m_cmap->GetColorRamp();
		(*m_currpoint)->mode = fn;
		m_requireupdate = true;
		canvas->Refresh();
	}
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::SetCMAPMode(CMAPMode mode)
{
	if (mode == m_mode)
		return;

	m_mode = mode;
	if (m_mode == MODE_GREY) {
		if (m_currcmpnt == 1 || m_currcmpnt == 2) {
			m_currcmpnt = 0;
		}
	}
	canvas->Refresh();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void CColorMapEditor::Pick(float x, float y, PickObject *obj)
{
	int cmpnt, i, priority[4];
	float val;
	ColorRamp *ramp	= m_cmap->GetColorRamp();
	ColorRamp::RampPoint::Tangent *tg;
	ColorRamp::PointListIter upointIter;
	ColorRamp::RampPoint *upoint;
	float pickrad;

	obj->type = UNKNOWN;

	// One works in priority on the definis objects by the user
	for (i = 0; i < m_currcmpnt; i++)
		priority[i] = i;
	for (i = m_currcmpnt + 1; i < 4; i++)
		priority[i - 1] = i;
	priority[3] = m_currcmpnt;

	pickrad = 5.0f;
	cmpnt = priority[3];
	for (i = 3; i >= 0; --i)
	{
		cmpnt = priority[i];
		if ((m_mode == MODE_GREY) && ((cmpnt == 1) || (cmpnt == 2)))
			continue;

		upointIter = ramp->Begin(cmpnt);
		while (upointIter != ramp->End(cmpnt))
		{
			upoint = *upointIter;

			// The POINT is sought
			if ( (fabs(y - upoint->point.y) < pickrad) &&
				 (fabs(x - upoint->point.x) < pickrad) )
			{
				obj->upoint = upointIter;
				obj->cmpnt = cmpnt;
				obj->type = POINT;
				return;
			}
			// The LEFT semi-tangent is sought
			tg = &upoint->tangent;
			if (fabs(x - tg->x0) < pickrad &&
				fabs(y - tg->y0) < pickrad)
			{
				obj->upoint = upointIter;
				obj->cmpnt = cmpnt;
				obj->type = LEFT_TAN;
				return;
			}
			// The RIGHT semi-tangent is sought
			if (fabs(x - tg->x1) < pickrad &&
				fabs(y - tg->y1) < pickrad)
			{
				obj->upoint = upointIter;
				obj->cmpnt = cmpnt;
				obj->type = RIGHT_TAN;
				return;
			}
			++upointIter;
		}
		/*
		Then one seeks the coordonnees on CURVE(On considering the precision 
		sufficient for the picking along the axis X) 
		Remark:  one does not leave in this case, because the priority is on 
		the users objects.
		*/
		upointIter = ramp->Begin(cmpnt) + 1;
		upoint = *upointIter;
		while (upoint->point.x < x)
		{
			upoint = *upointIter;
			if (upointIter == ramp->End(cmpnt))
				return;
			++upointIter;
		}
		int ndx = ((x - (int) x) < .5) ? (int) x : (int) x + 1;
		val = 255.0f * ramp->Evaluate(ndx, cmpnt);
		if (fabs(y - val) < pickrad)
		{
			obj->cmpnt = cmpnt;
			obj->ndx = ndx;
			obj->type = CURVE;
			return;
		}
	}
}
//-----------------------------------------------------------------------------

// --- Glue code to connect this class to the dialog in tcl file ---

//-----------------------------------------------------------------------------
void appSetCMAPCurrCmpnt(Tcl_Interp *interp, int cmpnt)
{
	assert(cmpnt >= 1);
	assert(cmpnt <= 4);

	Tcl_Eval(interp, "ResetCmpntButtons");
	std::ostringstream oss;
	oss << "SelectCurrCmpnt " << cmpnt;
	Tcl_Eval(interp, const_cast<char*>(oss.str().c_str()));
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void appSetCMAPCurrPtMode(Tcl_Interp *interp, ColorRamp::RampFunction fn)
{
	Tcl_Eval(interp, "ResetFunctionButtons");
	std::ostringstream oss;
	oss << "[W ";
	switch (fn) {
	case ColorRamp::CONSTANT:	oss << "b_Const";	break;
	case ColorRamp::LINEAR:		oss << "b_Linear";	break;
	case ColorRamp::POLYNOMIAL:	oss << "b_Spline";	break;
	}
	oss << "] configure -relief sunken -bg #aaaaaa -activebackground #aaaaaa";
	Tcl_Eval(interp, const_cast<char*>(oss.str().c_str()));
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void appDisableFunctionButtons(Tcl_Interp *interp)
{
	std::ostringstream oss;
	oss << "foreach i { b_Const b_Linear b_Spline } {"
		<< "	[W $i] configure -relief raised -state disabled"
		<< "			-bg $SystemButtonFace"
		<< "}";
	Tcl_Eval(interp, const_cast<char*>(oss.str().c_str()));
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void appDisableDeleteButton(Tcl_Interp *interp)
{
	std::ostringstream oss;
	oss << "[W b_RemovePt] configure -relief raised -state disabled"
		<< "			-bg $SystemButtonFace";
	Tcl_Eval(interp, const_cast<char*>(oss.str().c_str()));
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
void appEnableDeleteButton(Tcl_Interp *interp)
{
	std::ostringstream oss;
	oss << "[W b_RemovePt] configure -relief raised -state active"
		<< "			-bg $SystemButtonFace";
	Tcl_Eval(interp, const_cast<char*>(oss.str().c_str()));
}
//-----------------------------------------------------------------------------
