#include <CacheController.h>
#include <ILoadListener.h>
#include <CacheManager.h>
#include <PrefetchController.h>
#include <IReplacementVisitor.h>
#include <Iterator.h>
#include <BrushStore.h>
#include <ICacheController.h>
#include <log/log.h>
#include <ICacheEntry.h>
#include <DataIterator.h>
#include <RegionFactory.h>



using namespace cache;

/*
*	CacheController provides an interface to the Cache
*  controls the actions of the prefetcher.
*  Comments: if(pc!= NULL) is not an elegant solution a better design would
*  be to have the 2 versions of the class one with prefetcher and the other
*  without prefetcher, then have a common pure virutal class on top of it.
*  But I am choosing this way for efficiency purposes
*/

	/**
	* Constrcuts a new cache controller
	* \param PrefetchController 
	* \param *pc 
	* \return 
	*/
	CacheController ::CacheController(CacheManager* cacheManager,
		const unsigned int numberOfDimensions,					
		BrushStore* brushStore,
		PerformanceStatistics* perStatistics,
		PrefetchController *pc):
		cacheManager(cacheManager),brushStore(brushStore)

	{		
		this->pc = pc;
		this->perStatistics = perStatistics;
		this->numberOfDimensions = numberOfDimensions;
	}

	//Returns the current prefetch controller used only for experimental purposes
	PrefetchController* CacheController::getPrefetchController()
	{
		return(this->pc);
	}

	/**
	* Adds the listener to listen to the start and stop load events
	* \param loadListener 
	*/
	void CacheController:: addLoadListener(ILoadListener* loadListener)
	{
		this->vecLoadListener.push_back(loadListener);
	}


	/**
	* Request from the Front End
	* \param e1 
	* \param e2 
	* \param lod 
	* \param type 
	* \return 
	*/
	::Iterator<ICacheEntry*>* CacheController::
		load(double e1,double e2,double lod,int type)
	{
		#ifdef _DEBUG
		LOG_GMSG(controllerLogger,"Got in Load of controller");
		#endif

		//Preempting the Prefetcher		
		if(pc != NULL)			
		{
			this->pc->preempt();
		}

		
		//Calling all the listeners that registered for this event		
		for(unsigned int i =0 ;i<vecLoadListener.size();i++)		
			vecLoadListener[i]->startLoad();		

		//Make a brush
		Brush brush(e1,e2,lod,numberOfDimensions,type);

		//Adds the new Brush to the brush store.
		this->brushStore->addBrush(&brush);

		AscRegionFactory *regionFactory = AscRegionFactory
			::getInstance();//This decides the order in which the remainder regions
		//will be fetched.

		CacheIterator* cacheIterator;
		#ifdef _DEBUG
		LOG_GMSG(controllerLogger,"Calling CacheManager::load in controller");
		#endif
		cacheIterator	=   cacheManager->load(brush,
									this,//IReplacementVisitor
									this,//IFragThresholdListener
									regionFactory,
									(*this->perStatistics));											

		return(new DataIterator(cacheIterator,this));
	}

	/**
	*
	* \return PerformanceStatistics*
	*/
	PerformanceStatistics* CacheController::getPerformanceStatistics()
	{
		return(this->perStatistics);
	};
	/**
	* Called by the DataIterator when it is being deleted
	*/
	void CacheController:: endLoad()
	{
		for(unsigned int i =0 ;i<vecLoadListener.size();i++)		
			vecLoadListener[i]->stopLoad();	

		if(pc != NULL)
		{			
			this->pc->start();
			//You release a semaphore.
		}
	}

	//implements the IReplacementVisitor interface
	/**
	* 
	* \param cacheEntry 
	* \return 
	*/
	bool CacheController ::chosenItem(ICacheEntry& cacheEntry)
	{
		return(true);
	}

	/*
	*	Implements the IFragThresholdVisitor Interface
	*  This is used to notify the cache controller that the number of
	*  fragments for the query has exceeded the limit.
	*  It returns true because no matter what the user request has to be served	
	*/
	bool CacheController::exceededFragThreshold()
	{
		return(true);
	}


	/*
	 *	Destroying the cache controller.
	 */
	CacheController ::~CacheController()
	{}

