#include <LRUReplacement.h>
#include <DoublyList.h>
#include <ServedItemVisitor.h>
#include <ReplacementPolicy.h>
#include <BrushVisitor.h>
#include <assert.h>
#include <CacheEntry.h>
#include <list>
#include <cache/Cache.h>

using namespace cache;

LRUReplacement :: LRUReplacement(Cache *cache,
								 CacheEntry<LRUReplacementValue>* buffer,
								 unsigned long sizeOfBuffer,
								 IIndex *index):
								 ReplacementPolicy(cache)//set the cache replacement policy
				  
{
	//buffer is declared in the super class and points to the cache buffer
	//Should use dynamic cast here	
	this->lBuffer = buffer;
	//Stores the size of the buffer
	this->sizeOfBuffer = sizeOfBuffer;		
	this->cache = cache;

	this->freeList = new DoublyList<CacheEntry<LRUReplacementValue>*>;
	this->lruList = new DoublyList<CacheEntry<LRUReplacementValue>*>;
	this->notAvailableList = new DoublyList<CacheEntry<LRUReplacementValue>*>;

	//Inserting all the nodes into the free list
	for(unsigned long i=0;i< this->sizeOfBuffer;i++)
	{
		DoublyList<CacheEntry<LRUReplacementValue> *>::Node* node,tempNode;

		CacheEntry<LRUReplacementValue>* cacheEntryPtr = &this->lBuffer[i];		

		//The node contains a pointer to the cacheEntry		
		node = new DoublyList<CacheEntry<LRUReplacementValue>*>::Node(cacheEntryPtr);
		//the cache entry contains the pointer to the node.
		cacheEntryPtr->replacementValue.nodePtr = node;					
		this->freeList->AppendNode(node);
	} // for(int i=0;i< this->sizeOfBuffer;i++)
}


void LRUReplacement ::checkForError()
{
	unsigned int sum = this->notAvailableList->count() + 
					   this->freeList->count() +
					   this->lruList->count();

	if(sum != this->sizeOfBuffer)
	{
		ReplacementException r;
		throw(r);
	}
}
/**
 * Returns the next victim entry that is going to be replaced
 * \return 
 */
ICacheEntry* LRUReplacement ::peek()
{
	CacheEntry<LRUReplacementValue>* pointer = NULL;
	if(!freeList->isEmpty())	
	{
		pointer	= freeList->getFirst()->nData;
	}
	else if(!lruList->isEmpty())	
	{ 	
		pointer = lruList->getFirst()->nData;
	}
	return(pointer);
}


ICacheEntry* LRUReplacement ::getVictimItem()
{
	CacheEntry<LRUReplacementValue>* pointer = NULL;
	if(!freeList->isEmpty())	
	{
		pointer	= freeList->getFirst()->nData;
		//Removing the node from the list
		freeList->RemoveNode(pointer->replacementValue.nodePtr);
	}
	else if(!lruList->isEmpty())	
	{ 	
		pointer = lruList->getFirst()->nData;
		//Removing the node from the list
		lruList->RemoveNode(pointer->replacementValue.nodePtr);
		//Have to remove the node from the index in the cache
		this->cache->emptyCacheEntry(*pointer);
	}

	return(pointer);
}

//Note each call to getVictim should follow each call to newData

void LRUReplacement ::newData(ICacheEntry &cacheEntry)
{
//	CacheEntry<LRUReplacementValue>* entry = 
//		(CacheEntry<LRUReplacementValue>*) &cacheEntry;
	//Another way to avoid the type cast
	CacheEntry<LRUReplacementValue>* entry = &this->lBuffer[cacheEntry.getID()];

	this->notAvailableList->AppendNode(entry->replacementValue.nodePtr);
}

void LRUReplacement :: servedItem(ICacheEntry& cacheEntry)
{
//	CacheEntry<LRUReplacementValue>* entry = 
//		(CacheEntry<LRUReplacementValue>*) &cacheEntry;

	//Another way to avoid the type cast
	CacheEntry<LRUReplacementValue>* entry = &this->lBuffer[cacheEntry.getID()];

	//Remove from notAvailaible list 
	this->notAvailableList->RemoveNode(entry->replacementValue.nodePtr);
	//Insert into lrulist
	this->lruList->AppendNode(entry->replacementValue.nodePtr);
}

void LRUReplacement :: hitCacheEntry(ICacheEntry& cacheEntry)
{
	//CacheEntry<LRUReplacementValue>* entry = 
	//	(CacheEntry<LRUReplacementValue>*) &cacheEntry;

	//Another way to avoid the type cast
	CacheEntry<LRUReplacementValue>* entry = &this->lBuffer[cacheEntry.getID()];

	//When it is a hit the cacheEntry has to be in the lruList
	//Note remove node does not delete the node, it only removes it from the list
	this->lruList->RemoveNode(entry->replacementValue.nodePtr);
	//Append the same node to the end of the list
	this->notAvailableList->AppendNode(entry->replacementValue.nodePtr);
}


/**
* This call removes the list of cache entries
* from the cache. Used in case there are many or clauses
* in the remainder query.
* Removes the entries from the LRU list and adds them to the 
* free list.
* \param listCacheEntry 
*/
void LRUReplacement ::removeEntries(std::list<ICacheEntry*>& listCacheEntry)
{	
	std::list<ICacheEntry*>::const_iterator it;

	for (it=listCacheEntry.begin(); it != listCacheEntry.end(); ++it) {
			
			CacheEntry<LRUReplacementValue>* entry  = 
											&this->lBuffer[(*it)->getID()];
		    //Remove the entry from the lrulist.
			this->lruList->RemoveNode(entry->replacementValue.nodePtr);
			//Add that the node to the free list.
			this->freeList->AppendNode(entry->replacementValue.nodePtr);
			//Remove the cache entry from the index structure
			this->cache->emptyCacheEntry(*entry);
	}	
}


/**
 * This method is used only when the cache gets really fragmented.
 * That is the number of remainder regions exceeds a particular limit.
 * This would happen in cases where the size of the brush will be greater 
 * than the size of the cache.
 */
void LRUReplacement :: clearCache()
{	
		CacheEntry<LRUReplacementValue>* pointer = NULL;		
	    while(!lruList->isEmpty())	
		{ 	
			pointer = lruList->getFirst()->nData;
			//Removing the node from the list
			lruList->RemoveNode(pointer->replacementValue.nodePtr);
			//Have to remove the node from the index in the cache
			this->cache->emptyCacheEntry(*pointer);
			//Append the node to the free list of nodes
			this->freeList->AppendNode(pointer->replacementValue.nodePtr);
			//Note the pointer->replacemntValue.nodePtr is the same as
			//lruList->getFirst();
		} // while(!lruList->isEmpty())

		while(!notAvailableList->isEmpty())
		{
			pointer = notAvailableList->getFirst()->nData;
			//Removing the node from the list
			notAvailableList->RemoveNode(pointer->replacementValue.nodePtr);
			//Have to remove the node from the index in the cache
			this->cache->emptyCacheEntry(*pointer);
			//Append the node to the free list of nodes
			this->freeList->AppendNode(pointer->replacementValue.nodePtr);
			//Note the pointer->replacemntValue.nodePtr is the same as
			//notAvailableList->getFirst();
		} // while(!notAvailableList->isEmpty())
	//After this method all the cache entries will be free
}

void LRUReplacement :: brushModified(Brush& brush) 
{

}

void LRUReplacement::ifRemainder(bool flag)
{
	
}

LRUReplacement :: ~LRUReplacement()
{
	delete(this->freeList);
	delete(this->lruList);
	delete(this->notAvailableList);
}

